'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var MOBILE_DOC_VERSION = '0.3.1';
var MARKUP_SECTION_TYPE = 1;
var IMAGE_SECTION_TYPE = 2;
var LIST_SECTION_TYPE = 3;
var CARD_SECTION_TYPE = 10;
var MARKUP_MARKER_TYPE = 0;
var ATOM_MARKER_TYPE = 1;
var defaultOptions = {
  atoms: [],
  cards: [],
  markups: [],
  sections: [],
  additionalProps: []
  /**
   * React Hook to render a mobiledoc.
   * @param {*} mobiledoc
   * @param {*} options
   */

};
function useMobiledoc(mobiledoc, options) {
  var _defaultOptions$optio = _objectSpread({}, defaultOptions, options),
      atoms = _defaultOptions$optio.atoms,
      cards = _defaultOptions$optio.cards,
      markups = _defaultOptions$optio.markups,
      sections = _defaultOptions$optio.sections,
      additionalProps = _defaultOptions$optio.additionalProps;
  /**
   * Looks for an item in array of objects with name
   *
   * @param {Array} items list of objects to look through
   * @param {String} name name to match
   */

  function findByName(items, name) {
    return items.find(function (i) {
      return i.name === name;
    });
  }
  /**
   * Render a Markup (inline markup, like strong, em, etc)
   *
   * @param {Array} section Markup section's data
   * @param {Number} nodeKey Section index
   */


  function renderMarkupSection(_ref, nodeKey) {
    var _ref2 = _slicedToArray(_ref, 3),
        type = _ref2[0],
        Tag = _ref2[1],
        markers = _ref2[2];

    var Element;
    var customSection = findByName(sections, Tag);

    if (customSection) {
      var Section = customSection.component;
      Element = React.createElement(Section, _extends({
        key: nodeKey
      }, additionalProps), []);
    } else {
      Element = React.createElement(Tag, {
        key: nodeKey
      }, []);
    }

    return renderMarkersOnElement(Element, markers, nodeKey);
  }
  /**
   * Render a List (ul / ol)
   * @param {Array} section Markup section's data
   * @param {Number} nodeKey Section index
   */


  function renderListSection(_ref3, nodeKey) {
    var _ref4 = _slicedToArray(_ref3, 3),
        type = _ref4[0],
        Tag = _ref4[1],
        markers = _ref4[2];

    var items = markers.map(function (item, key) {
      var itemKey = "".concat(nodeKey, "-").concat(key);
      return renderMarkersOnElement(React.createElement("li", {
        key: itemKey
      }, []), item, itemKey);
    });
    return React.createElement(Tag, {
      key: nodeKey
    }, items);
  }
  /**
   * Renders an Atom
   * @param {Number} atomIndex Atom Index
   */


  function renderAtomSection(atomIndex) {
    var _mobiledoc$atoms$atom = _slicedToArray(mobiledoc.atoms[atomIndex], 3),
        name = _mobiledoc$atoms$atom[0],
        text = _mobiledoc$atoms$atom[1],
        payload = _mobiledoc$atoms$atom[2];

    var atom = findByName(atoms, name);

    if (atom) {
      var props = {
        env: {
          name: name,
          isInEditor: false,
          dom: 'dom'
        },
        options: {},
        key: "".concat(name, "-").concat(text.length),
        payload: _objectSpread({}, payload, additionalProps),
        text: text
      };
      return atom.component(props);
    }

    return null;
  }
  /**
   * Renders a Card section
   * @param {Array} section Card section's data
   * @param {Number} nodeKey Section index
   */


  function renderCardSection(_ref5, nodeKey) {
    var _ref6 = _slicedToArray(_ref5, 2),
        type = _ref6[0],
        index = _ref6[1];

    var _mobiledoc$cards$inde = _slicedToArray(mobiledoc.cards[index], 2),
        name = _mobiledoc$cards$inde[0],
        payload = _mobiledoc$cards$inde[1];

    var card = findByName(cards, name);

    if (card) {
      var props = {
        env: {
          name: name,
          isInEditor: false,
          dom: 'dom',
          didRender: function didRender(cb) {
            return registerRenderCallback(cb);
          },
          onTeardown: function onTeardown(cb) {
            return registerRenderCallback(cb);
          }
        },
        options: {},
        payload: _objectSpread({}, payload, additionalProps),
        key: nodeKey
      };
      return card.component(props);
    }

    return null;
  }
  /**
   * Renders element's inner markers
   * @param {ReactElement} element
   * @param {Array} markers
   * @param {Number} parentKey
   */


  function renderMarkersOnElement(element, markers, parentKey) {
    var elements = [element];

    function pushElement(openedElement) {
      element.props.children.push(openedElement);
      elements.push(openedElement);
      element = openedElement;
    }

    for (var markerKey in markers) {
      var _markers$markerKey = _slicedToArray(markers[markerKey], 4),
          type = _markers$markerKey[0],
          openTypes = _markers$markerKey[1],
          closeCount = _markers$markerKey[2],
          value = _markers$markerKey[3];

      for (var openTypeKey in openTypes) {
        var openType = openTypes[openTypeKey];

        var _mobiledoc$markups$op = _slicedToArray(mobiledoc.markups[openType], 2),
            Tag = _mobiledoc$markups$op[0],
            attrs = _mobiledoc$markups$op[1];

        if (Tag) {
          var props = _objectSpread({
            children: []
          }, parseProps(attrs), {
            key: "".concat(parentKey, "-").concat(markerKey, "-").concat(openTypeKey)
          });

          var definedMarkup = findByName(markups, Tag);

          if (definedMarkup) {
            var Markup = definedMarkup.component;
            pushElement(React.createElement(Markup, props));
          } else {
            pushElement(React.createElement(Tag, props));
          }
        } else {
          closeCount -= 1;
        }
      }

      switch (type) {
        case MARKUP_MARKER_TYPE:
          element.props.children.push(value);
          break;

        case ATOM_MARKER_TYPE:
          element.props.children.push(renderAtomSection(value));
          break;

        default:
      }

      for (var j = 0, m = closeCount; j < m; j += 1) {
        elements.pop();
        element = elements[elements.length - 1];
      }
    }

    return element;
  }
  /**
   * Converts attrs to props
   *
   * @param {array} attrs
   */


  function parseProps(attrs) {
    if (attrs) {
      return _defineProperty({}, attrs[0], attrs[1]);
    }

    return null;
  }
  /**
   * Register a render callback (`didRender`/`onTeardown`)
   * @param {Function} cb Callback function
   */


  function registerRenderCallback(cb) {
  } // Returns/Renders all sections in mobiledoc


  return mobiledoc.sections.map(function (section, key) {
    var _section = _slicedToArray(section, 1),
        type = _section[0];

    var nodeKey = "mobiledoc-".concat(type, "-").concat(key);

    switch (type) {
      case MARKUP_SECTION_TYPE:
        return renderMarkupSection(section, nodeKey);

      case LIST_SECTION_TYPE:
        return renderListSection(section, nodeKey);

      case CARD_SECTION_TYPE:
        return renderCardSection(section, nodeKey);

      default:
        return null;
    }
  }).filter(Boolean); // Remove null items from array
}

exports.MOBILE_DOC_VERSION = MOBILE_DOC_VERSION;
exports.MARKUP_SECTION_TYPE = MARKUP_SECTION_TYPE;
exports.IMAGE_SECTION_TYPE = IMAGE_SECTION_TYPE;
exports.LIST_SECTION_TYPE = LIST_SECTION_TYPE;
exports.CARD_SECTION_TYPE = CARD_SECTION_TYPE;
exports.MARKUP_MARKER_TYPE = MARKUP_MARKER_TYPE;
exports.ATOM_MARKER_TYPE = ATOM_MARKER_TYPE;
exports.default = useMobiledoc;
//# sourceMappingURL=useMobiledoc.js.map
